@@ -2,7 +2,8 @@ import sys
 
 import matplotlib
 import matplotlib.style as mplstyle
-mplstyle.use('fast')
+
+mplstyle.use("fast")
 
 matplotlib.use("Agg")
 
@@ -17,31 +18,28 @@ matplotlib.rc("legend", fontsize=size)  # legend fontsize
 matplotlib.rc("figure", titlesize=size)  # fontsize of the figure title
 matplotlib.rc("figure", autolayout=True)
 
+import itertools
+import logging
+
+# parallelization
+import multiprocessing
+import os
+import warnings
+from pathlib import Path
+
+import numpy
+import scipy.interpolate
+from addict import Dict
+from cadet import H5, Cadet
+from emcee import autocorr
 from matplotlib import figure
 from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
 from mpl_toolkits.mplot3d import Axes3D
 
+import CADETMatch.loggerwriter as loggerwriter
+import CADETMatch.util as util
 from CADETMatch.cache import cache
 
-from pathlib import Path
-import pandas
-import numpy
-import scipy.interpolate
-import itertools
-
-from cadet import Cadet, H5
-from addict import Dict
-
-# parallelization
-import multiprocessing
-
-import os
-import warnings
-import CADETMatch.util as util
-import logging
-import CADETMatch.loggerwriter as loggerwriter
-
-from emcee import autocorr
 
 def main(map_function):
     cache.setup_dir(sys.argv[1])
@@ -53,9 +51,15 @@ def main(map_function):
     mcmcDir = Path(cache.settings["resultsDirMCMC"])
     mcmc_h5 = mcmcDir / "mcmc.h5"
     if mcmc_h5.exists():
+        resultDir = Path(cache.settings["resultsDirBase"])
+        result_lock = resultDir / "result.lock"
+
         mcmc_store = H5()
         mcmc_store.filename = mcmc_h5.as_posix()
-        mcmc_store.load(paths=["/full_chain", "/train_full_chain", "/bounds_full_chain"])
+        mcmc_store.load(
+            paths=["/full_chain", "/train_full_chain", "/bounds_full_chain"],
+            lock=True
+        )
 
         progress_path = Path(cache.settings["resultsDirBase"]) / "result.h5"
 
@@ -65,7 +69,9 @@ def main(map_function):
 
         for chain in ("full_chain", "train_full_chain", "bounds_full_chain"):
             if chain in mcmc_store.root:
-                plot_chain(input_headers, mcmc_store.root[chain], chain, graph_dir / chain)
+                plot_chain(
+                    input_headers, mcmc_store.root[chain], chain, graph_dir / chain
+                )
 
 
 def plot_chain(headers, chain, chain_name, graph_dir):
