@@ -1,45 +1,42 @@
-import numpy
-from sklearn import preprocessing
-from sklearn.neighbors import KernelDensity
-import scipy
 import multiprocessing
 import sys
-
-import cadet
-import CADETMatch.util as util
-import CADETMatch.evo as evo
-import pandas
-from addict import Dict
-from CADETMatch.cache import cache
-import CADETMatch.smoothing as smoothing
-import CADETMatch.kde_util as kde_util
-from pathlib import Path
 import warnings
+from pathlib import Path
+
 import joblib
-
-with warnings.catch_warnings():
-    warnings.filterwarnings("ignore", category=FutureWarning)
-    import h5py
-
 import matplotlib
 import matplotlib.style as mplstyle
-mplstyle.use('fast')
+import numpy
+import pandas
+import scipy
+from addict import Dict
+from cadet import H5, Cadet
+from sklearn import preprocessing
+from sklearn.neighbors import KernelDensity
+
+import CADETMatch.evo as evo
+import CADETMatch.kde_util as kde_util
+import CADETMatch.smoothing as smoothing
+import CADETMatch.util as util
+from CADETMatch.cache import cache
+
+mplstyle.use("fast")
 
 matplotlib.use("Agg")
 
+import matplotlib.cm
+import matplotlib.pyplot as plt
 from matplotlib import figure
 from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
 
-import matplotlib.pyplot as plt
-
-import matplotlib.cm
-
 cm_plot = matplotlib.cm.gist_rainbow
 
+import itertools
 import logging
+
 import CADETMatch.loggerwriter as loggerwriter
 
-import itertools
+import arviz as az
 
 
 def get_color(idx, max_colors, cmap):
@@ -65,18 +62,33 @@ plt.rc("legend", fontsize=size)  # legend fontsize
 plt.rc("figure", titlesize=size)  # fontsize of the figure title
 plt.rc("figure", autolayout=True)
 
-atol = 1e-3
-rtol = 1e-3
+class ArvizSampler:
+    def __init__(self, chain, prob):
+        self.chain = chain.swapaxes(0, 1)
+        self.prob = prob.swapaxes(0, 1)
+    
+    def get_chain(self):
+        return self.chain
 
+    def get_log_prob(self):
+        return self.prob
 
-def reduce_data(data, size, bw_size):
-    # size reduces data for normal usage, bw_size reduces the size just for bandwidth estimation
-    lb, ub = numpy.percentile(data, [5, 95], 0)
-    selected = (data >= lb) & (data <= ub)
+def flatten(chain):
+    chain_shape = chain.shape
+    flat_chain = chain.reshape(chain_shape[0] * chain_shape[1], chain_shape[2])
+    return flat_chain
 
-    selected = numpy.all(selected, 1)
+def reduce_data(data_chain, probability, headers, size, bw_size):
+    sampler = ArvizSampler(data_chain, probability)
+    emcee_data = az.from_emcee(sampler)
+    hdi = az.hdi(emcee_data, hdi_prob=0.9).to_array().values
 
-    data = data[selected, :]
+    data= flatten(data_chain)
+
+    selected = (data > hdi[:,0]) & (data < hdi[:,1])
+    selected = numpy.all(selected, axis=1)
+
+    data = data[selected]
 
     scaler = preprocessing.RobustScaler().fit(data)
 
@@ -104,30 +116,54 @@ def goal_kde(x, kde):
     score = kde.score_samples(test_value)
     return -score[0]
 
-def get_mle(data):
+
+def get_mle(data_chain, probability, headers):
+    popsize = 100
+
     multiprocessing.get_logger().info("setting up scaler and reducing data")
-    data, data_reduced, data_reduced_bw, scaler = reduce_data(data, 32000, 10000)
+    data_transform, data_reduced, data_reduced_bw, scaler = reduce_data(
+        data_chain, probability, headers, 32000, 20000
+    )
     multiprocessing.get_logger().info("finished setting up scaler and reducing data")
     multiprocessing.get_logger().info("data_reduced shape %s", data_reduced.shape)
 
-    BOUND_LOW_num = numpy.min(data, 0)
-    BOUND_UP_num = numpy.max(data, 0)
+    BOUND_LOW_num = numpy.min(data_transform, 0)
+    BOUND_UP_num = numpy.max(data_transform, 0)
 
     BOUND_LOW_trans = list(BOUND_LOW_num)
     BOUND_UP_trans = list(BOUND_UP_num)
 
-    kde_ga = KernelDensity(kernel="gaussian", atol=atol, rtol=rtol)
+    kde_ga = KernelDensity(kernel="gaussian")
 
     kde_ga, bandwidth, store = kde_util.get_bandwidth(kde_ga, data_reduced_bw)
 
     multiprocessing.get_logger().info("mle bandwidth: %.2g", bandwidth)
 
     multiprocessing.get_logger().info("fitting kde with mle bandwidth")
-    kde_ga.fit(data)
+    kde_ga.fit(data_reduced)
     multiprocessing.get_logger().info("finished fitting and starting mle search")
 
+    probability_ln = kde_ga.score_samples(data_transform)
+
+    idx_max_ln = numpy.argmax(probability_ln)
+    prob_best_ln = probability_ln[idx_max_ln]
+    best_point = data_transform[idx_max_ln].reshape(1, -1)
+
+    multiprocessing.get_logger().info(
+        "starting point %s=%s", data_transform[idx_max_ln], -prob_best_ln
+    )
+
+    individuals_mle = kde_ga.sample(popsize - 1)
+
+    init = numpy.concatenate([best_point, individuals_mle])
+
     result_kde = scipy.optimize.differential_evolution(
-        goal_kde, bounds=list(zip(BOUND_LOW_trans, BOUND_UP_trans)), args=(kde_ga,), disp=True, popsize=100
+        goal_kde,
+        bounds=list(zip(BOUND_LOW_trans, BOUND_UP_trans)),
+        args=(kde_ga,),
+        disp=True,
+        popsize=popsize,
+        init=init,
     )
 
     multiprocessing.get_logger().info("finished mle search")
@@ -151,28 +187,33 @@ def addChain(axis, *args):
         return numpy.array(temp[0])
 
 
-def process_mle(chain, gen, cache):
+def process_mle(chain, probability, gen, cache):
     mcmc_dir = Path(cache.settings["resultsDirMCMC"])
 
     mcmc_csv = Path(cache.settings["resultsDirMCMC"]) / "prob.csv"
     mle_h5 = Path(cache.settings["resultsDirMCMC"]) / "mle.h5"
 
-    h5 = cadet.H5()
+    h5 = H5()
     h5.filename = mle_h5.as_posix()
     if mle_h5.exists():
-        h5.load()
+        h5.load(lock=True)
 
-        if 0:  # h5.root.generations[-1] == gen:
-            multiprocessing.get_logger().info("new information is not yet available and mle will quit")
+        if 0: #h5.root.generations[-1] == gen:
+            multiprocessing.get_logger().info(
+                "new information is not yet available and mle will quit"
+            )
             return
 
-    multiprocessing.get_logger().info("process mle chain shape before %s", chain.shape)
+    #multiprocessing.get_logger().info("process mle chain shape before %s", chain.shape)
     # This step cleans up bad entries
-    chain = chain[~numpy.all(chain == 0, axis=1)]
-    multiprocessing.get_logger().info("process mle chain shape after cleaning 0 entries %s", chain.shape)
+    #selected = ~numpy.all(chain == 0, axis=1)
+    #chain = chain[selected]
+    #probability = probability[selected]
+    #multiprocessing.get_logger().info(
+    #    "process mle chain shape after cleaning 0 entries %s", chain.shape
+    #)
 
-
-    mle_x, kde, scaler = get_mle(chain)
+    mle_x, kde, scaler = get_mle(chain, probability, cache.parameter_headers_actual)
 
     multiprocessing.get_logger().info("mle_x: %s", mle_x)
 
@@ -180,7 +221,7 @@ def process_mle(chain, gen, cache):
     joblib.dump(scaler, mcmcDir / "kde_prior_scaler.joblib")
     joblib.dump(kde, mcmcDir / "kde_prior.joblib")
 
-    mle_ind = util.convert_individual(mle_x, cache)[0]
+    mle_ind = util.convert_individual_inputorder(mle_x, cache)
 
     multiprocessing.get_logger().info("mle_ind: %s", mle_ind)
 
@@ -192,17 +233,19 @@ def process_mle(chain, gen, cache):
 
     # run simulations for 5% 50% 95% and MLE vs experimental data
     percentile_splits = [5, 10, 50, 90, 95]
-    percentile = numpy.percentile(chain, percentile_splits, 0)
+    percentile = numpy.percentile(flatten(chain), percentile_splits, 0)
 
     multiprocessing.get_logger().info("percentile: %s %s", percentile.shape, percentile)
 
     for row in percentile:
         temp.append(list(row))
 
-    cadetValues = [util.convert_individual(i, cache)[0] for i in temp]
+    cadetValues = [util.convert_individual_inputorder(i, cache) for i in temp]
     cadetValues = numpy.array(cadetValues)
 
-    multiprocessing.get_logger().info("cadetValues: %s %s", cadetValues.shape, cadetValues)
+    multiprocessing.get_logger().info(
+        "cadetValues: %s %s", cadetValues.shape, cadetValues
+    )
 
     map_function = util.getMapFunction()
 
@@ -216,7 +259,9 @@ def process_mle(chain, gen, cache):
 
             simulations[name] = sims
 
-    multiprocessing.get_logger().info("type %s  value %s", type(cadetValues), cadetValues)
+    multiprocessing.get_logger().info(
+        "type %s  value %s", type(cadetValues), cadetValues
+    )
 
     pd = pandas.DataFrame(cadetValues, columns=cache.parameter_headers_actual)
     labels = ["MLE", "5", "10", "50", "90", "95"]
@@ -230,10 +275,16 @@ def process_mle(chain, gen, cache):
 
     h5.root.mles = addChain(1, h5.root.mles, mle_x[:, numpy.newaxis])
     h5.root.stats = addChain(2, h5.root.stats, percentile[:, :, numpy.newaxis])
-    h5.root.generations = addChain(0, h5.root.generations, [gen,])
+    h5.root.generations = addChain(
+        0,
+        h5.root.generations,
+        [
+            gen,
+        ],
+    )
     h5.root.stat_MLE = mle_ind
 
-    h5.save()
+    h5.save(lock=True)
 
     plot_mle(simulations, cache, labels)
 
@@ -251,14 +302,9 @@ def plot_mle(simulations, cache, labels):
         units_used = cache.target[experimentName]["units_used"]
 
         numPlotsSeq = [len(units_used)]
-        # Shape and ShapeDecay have a chromatogram + derivative
-        for feature in experiment["features"]:
-            if feature["type"] in ("Shape", "ShapeDecay", "ShapeFront", "ShapeBack"):
-                numPlotsSeq.append(2)
-            elif feature["type"] in ("AbsoluteTime", "AbsoluteHeight"):
-                pass
-            else:
-                numPlotsSeq.append(1)
+        for feature in experiment["scores"]:
+            settings = cache.scores[feature["type"]].get_settings(feature)
+            numPlotsSeq.append(settings.graph_der + settings.graph + settings.graph_frac)
 
         numPlots = sum(numPlotsSeq)
 
@@ -269,10 +315,15 @@ def plot_mle(simulations, cache, labels):
         canvas = FigureCanvas(fig)
 
         for idx, unit in enumerate(units_used):
-            graph_simulations(simulations[experimentName], labels, unit, fig.add_subplot(numPlots, 1, idx + 1))
+            graph_simulations(
+                simulations[experimentName],
+                labels,
+                unit,
+                fig.add_subplot(numPlots, 1, idx + 1),
+            )
 
         graphIdx = 2
-        for idx, feature in enumerate(experiment["features"]):
+        for idx, feature in enumerate(experiment["scores"]):
             featureName = feature["name"]
             featureType = feature["type"]
 
@@ -282,41 +333,18 @@ def plot_mle(simulations, cache, labels):
             exp_time = feat["time"][selected]
             exp_value = feat["value"][selected]
 
-            if featureType in (
-                "similarity",
-                "similarityDecay",
-                "similarityHybrid",
-                "similarityHybrid2",
-                "similarityHybrid2_spline",
-                "similarityHybridDecay",
-                "similarityHybridDecay2",
-                "curve",
-                "breakthrough",
-                "dextran",
-                "dextranHybrid",
-                "dextranHybrid2",
-                "dextranHybrid2_spline",
-                "similarityCross",
-                "similarityCrossDecay",
-                "breakthroughCross",
-                "SSE",
-                "LogSSE",
-                "breakthroughHybrid",
-                "breakthroughHybrid2",
-                "Shape",
-                "ShapeDecay",
-                "Dextran",
-                "DextranAngle",
-                "DextranTest",
-                "ShapeDecaySimple",
-                "ShapeFront",
-                "ShapeBack",
-            ):
+            settings = cache.scores[featureType].get_settings(feature)
+            if settings.graph:
+                graph = fig.add_subplot(
+                    numPlots, 1, graphIdx
+                )  # additional +1 added due to the overview plot
 
-                graph = fig.add_subplot(numPlots, 1, graphIdx)  # additional +1 added due to the overview plot
-
-                for idx, (sim, label) in enumerate(zip(simulations[experimentName], labels)):
-                    sim_time, sim_value = util.get_times_values(sim, target[experimentName][featureName])
+                for idx, (sim, label) in enumerate(
+                    zip(simulations[experimentName], labels)
+                ):
+                    sim_time, sim_value = util.get_times_values(
+                        sim, target[experimentName][featureName]
+                    )
 
                     if idx == 0:
                         linewidth = 2
@@ -328,7 +356,9 @@ def plot_mle(simulations, cache, labels):
                         sim_value,
                         "--",
                         label=label,
-                        color=get_color(idx, len(simulations[experimentName]) + 1, cm_plot),
+                        color=get_color(
+                            idx, len(simulations[experimentName]) + 1, cm_plot
+                        ),
                         linewidth=linewidth,
                     )
 
@@ -337,49 +367,32 @@ def plot_mle(simulations, cache, labels):
                     exp_value,
                     "-",
                     label="Experiment",
-                    color=get_color(len(simulations[experimentName]), len(simulations[experimentName]) + 1, cm_plot),
+                    color=get_color(
+                        len(simulations[experimentName]),
+                        len(simulations[experimentName]) + 1,
+                        cm_plot,
+                    ),
                     linewidth=2,
                 )
                 graphIdx += 1
 
-            if featureType in (
-                "similarity",
-                "similarityDecay",
-                "similarityHybrid",
-                "similarityHybrid2",
-                "similarityHybrid2_spline",
-                "similarityHybridDecay",
-                "similarityHybridDecay2",
-                "curve",
-                "breakthrough",
-                "dextran",
-                "dextranHybrid",
-                "dextranHybrid2",
-                "dextranHybrid2_spline",
-                "similarityCross",
-                "similarityCrossDecay",
-                "breakthroughCross",
-                "SSE",
-                "LogSSE",
-                "breakthroughHybrid",
-                "breakthroughHybrid2",
-                "Shape",
-                "ShapeDecay",
-                "Dextran",
-                "DextranAngle",
-                "DextranTest",
-                "DextranQuad",
-                "Dextran3",
-                "DextranShape",
-                "ShapeFront",
-                "ShapeBack",
-            ):
+            if settings.graph_der:
 
-                graph = fig.add_subplot(numPlots, 1, graphIdx)  # additional +1 added due to the overview plot
-                for idx, (sim, label) in enumerate(zip(simulations[experimentName], labels)):
-                    sim_time, sim_value = util.get_times_values(sim, target[experimentName][featureName])
+                graph = fig.add_subplot(
+                    numPlots, 1, graphIdx
+                )  # additional +1 added due to the overview plot
+                for idx, (sim, label) in enumerate(
+                    zip(simulations[experimentName], labels)
+                ):
+                    sim_time, sim_value = util.get_times_values(
+                        sim, target[experimentName][featureName]
+                    )
                     sim_spline = smoothing.smooth_data_derivative(
-                        sim_time, sim_value, feat["critical_frequency"], feat["smoothing_factor"], feat["critical_frequency_der"]
+                        sim_time,
+                        sim_value,
+                        feat["critical_frequency"],
+                        feat["smoothing_factor"],
+                        feat["critical_frequency_der"],
                     )
 
                     if idx == 0:
@@ -392,19 +405,29 @@ def plot_mle(simulations, cache, labels):
                         sim_spline,
                         "--",
                         label=label,
-                        color=get_color(idx, len(simulations[experimentName]) + 1, cm_plot),
+                        color=get_color(
+                            idx, len(simulations[experimentName]) + 1, cm_plot
+                        ),
                         linewidth=linewidth,
                     )
 
                 exp_spline = smoothing.smooth_data_derivative(
-                    exp_time, exp_value, feat["critical_frequency"], feat["smoothing_factor"], feat["critical_frequency_der"]
+                    exp_time,
+                    exp_value,
+                    feat["critical_frequency"],
+                    feat["smoothing_factor"],
+                    feat["critical_frequency_der"],
                 )
                 graph.plot(
                     exp_time,
                     exp_spline,
                     "-",
                     label="Experiment",
-                    color=get_color(len(simulations[experimentName]), len(simulations[experimentName]) + 1, cm_plot),
+                    color=get_color(
+                        len(simulations[experimentName]),
+                        len(simulations[experimentName]) + 1,
+                        cm_plot,
+                    ),
                     linewidth=2,
                 )
                 graphIdx += 1
@@ -416,7 +439,9 @@ def plot_mle(simulations, cache, labels):
 
 def graph_simulations(simulations, simulation_labels, unit, graph):
     linestyles = ["-", "--", "-.", ":"]
-    for idx_sim, (simulation, label_sim) in enumerate(zip(simulations, simulation_labels)):
+    for idx_sim, (simulation, label_sim) in enumerate(
+        zip(simulations, simulation_labels)
+    ):
 
         comps = []
 
@@ -427,18 +452,32 @@ def graph_simulations(simulations, simulation_labels, unit, graph):
 
         solution_times = simulation.root.output.solution.solution_times
 
-        hasColumn = any("column" in i for i in simulation.root.output.solution[unit].keys())
+        hasColumn = any(
+            "column" in i for i in simulation.root.output.solution[unit].keys()
+        )
         hasPort = any("port" in i for i in simulation.root.output.solution[unit].keys())
 
         if hasColumn:
             for i in range(ncomp):
-                comps.append(simulation.root.output.solution[unit]["solution_column_outlet_comp_%03d" % i])
+                comps.append(
+                    simulation.root.output.solution[unit][
+                        "solution_column_outlet_comp_%03d" % i
+                    ]
+                )
         elif hasPort:
             for i in range(ncomp):
-                comps.append(simulation.root.output.solution[unit]["solution_outlet_port_000_comp_%03d" % i])
+                comps.append(
+                    simulation.root.output.solution[unit][
+                        "solution_outlet_port_000_comp_%03d" % i
+                    ]
+                )
         else:
             for i in range(ncomp):
-                comps.append(simulation.root.output.solution[unit]["solution_outlet_comp_%03d" % i])
+                comps.append(
+                    simulation.root.output.solution[unit][
+                        "solution_outlet_comp_%03d" % i
+                    ]
+                )
 
         if hasSalt:
             graph.set_title("Output")
@@ -491,10 +530,16 @@ def main():
     mcmcDir = Path(cache.settings["resultsDirMCMC"])
     mcmc_h5 = mcmcDir / "mcmc.h5"
 
-    with h5py.File(mcmc_h5, "r") as h5:
-        flat_chain = h5["/flat_chain"][()]
-        mcmc_acceptance = h5["/mcmc_acceptance"][()]
-        process_mle(flat_chain, len(mcmc_acceptance), cache)
+    resultDir = Path(cache.settings["resultsDirBase"])
+    result_lock = resultDir / "result.lock"
+
+    mcmc_store = H5()
+    mcmc_store.filename = mcmc_h5.as_posix()
+    mcmc_store.load(paths=["/full_chain", "/mcmc_acceptance", "/run_probability_flat"], lock=True)
+
+    process_mle(mcmc_store.root.full_chain, 
+                mcmc_store.root.run_probability_flat,
+                len(mcmc_store.root.mcmc_acceptance), cache)
 
 
 if __name__ == "__main__":
