@@ -46,12 +46,12 @@ def get_times_values(simulation, target, selected=None):
     try:
         times = simulation.root.output.solution.solution_times
 
-        isotherm = target["isotherm"]
+        output_path = target["output_path"]
 
-        if isinstance(isotherm, list):
-            values = numpy.sum([simulation[i] for i in isotherm], 0)
+        if isinstance(output_path, list):
+            values = numpy.sum([simulation[i] for i in output_path], 0)
         else:
-            values = simulation[isotherm]
+            values = simulation[output_path]
     except (AttributeError, KeyError):
         times = simulation[:, 0]
         values = simulation[:, 1]
@@ -136,6 +136,11 @@ def saveExperiments(save_name_base, settings, target, results, directory, file_p
     for experiment in settings["experiments"]:
         experimentName = experiment["name"]
         simulation = results[experimentName]["simulation"]
+        try:
+            #for addict 2.4.0 otherwise when variables are set below it can't find __frozen even though it is not used
+            simulation.root.unfreeze()
+        except TypeError:
+            pass
 
         dst = Path(directory, file_pattern % (save_name_base, experimentName))
 
@@ -311,11 +316,13 @@ def runExperiment(
     cache,
     post_function=None,
 ):
-    handle, path = tempfile.mkstemp(suffix=".h5", dir=cache.tempDir)
-    os.close(handle)
-
     simulation = Cadet(template_sim.root)
-    simulation.filename = path
+
+    if simulation.is_file:
+        handle, path = tempfile.mkstemp(suffix=".h5", dir=cache.tempDir)
+        os.close(handle)
+    
+        simulation.filename = path
 
     simulation.root.input.solver.nthreads = int(settings.get("nThreads", 1))
 
@@ -327,23 +334,30 @@ def runExperiment(
         cadetValues = []
         cadetValuesKEQ = []
 
-    simulation.save()
+    if simulation.is_file:
+        simulation.save()
 
     try:
         simulation.run(timeout=timeout, check=True)
     except subprocess.TimeoutExpired:
         multiprocessing.get_logger().warn("Simulation Timed Out")
-        os.remove(path)
+        simulation.clear()
+        if simulation.is_file:
+            os.remove(path)
         return None
 
     except subprocess.CalledProcessError as error:
         multiprocessing.get_logger().error("The simulation failed %s", individual)
         logError(cache, cadetValuesKEQ, error)
+        simulation.clear()
         return None
 
     # read sim data
-    simulation.load(paths=["/meta", "/output"], update=True)
-    os.remove(path)
+    simulation.load_results()
+    simulation.clear()
+
+    if simulation.is_file:
+        os.remove(path)
 
     simulationFailed = isinstance(simulation.root.output.solution.solution_times, Dict)
     if simulationFailed:
@@ -358,7 +372,7 @@ def runExperiment(
 
     temp = {}
     temp["simulation"] = simulation
-    temp["path"] = path
+    temp["path"] = path if simulation.is_file else None
     temp["scores"] = []
     temp["error"] = 0.0
     temp["error_count"] = 0.0
@@ -373,7 +387,7 @@ def runExperiment(
     temp["sim_value"] = []
     temp["exp_value"] = []
 
-    for feature in experiment["features"]:
+    for feature in experiment["scores"]:
         featureType = feature["type"]
         featureName = feature["name"]
 
@@ -473,22 +487,21 @@ def setupMCMC(cache):
             update_json_mcmc(settings)
 
         settings["searchMethod"] = "MCMC"
-        settings["graphSpearman"] = 0
 
         for experiment in settings["experiments"]:
             foundAbsoluteTime = False
             foundAbsoluteHeight = False
-            for feature in experiment["features"]:
+            for feature in experiment["scores"]:
                 if feature["type"] == "AbsoluteTime":
                     foundAbsoluteTime = True
                 if feature["type"] == "AbsoluteHeight":
                     foundAbsoluteHeight = True
             if foundAbsoluteTime is False:
-                experiment["features"].append(
+                experiment["scores"].append(
                     {"name": "AbsoluteTime", "type": "AbsoluteTime"}
                 )
             if foundAbsoluteHeight is False:
-                experiment["features"].append(
+                experiment["scores"].append(
                     {"name": "AbsoluteHeight", "type": "AbsoluteHeight"}
                 )
 
@@ -559,10 +572,10 @@ def setupAltFeature(cache, name):
         settings["searchMethod"] = "AltScore"
 
         for experiment in settings["experiments"]:
-            for feature in experiment["featuresAlt"]:
+            for feature in experiment["scoresAlt"]:
                 if feature["name"] == name:
-                    experiment["features"] = feature["features"]
-            del experiment["featuresAlt"]
+                    experiment["scores"] = feature["scores"]
+            del experiment["scoresAlt"]
 
         new_settings_file = resultDir / settings_file.name
         with new_settings_file.open(mode="w") as json_data:
@@ -599,7 +612,7 @@ def getBestIndividual(cache):
     results.filename = progress_path.as_posix()
     results.load(paths=["/meta_score", "/meta_population"], lock=True)
 
-    idx = numpy.argmax(results.root.meta_score[:, 1])
+    idx = numpy.argmin(results.root.meta_score[:, 0])
     individual = results.root.meta_population[idx, :]
     return individual
 
@@ -626,7 +639,7 @@ def copyCSVWithNoise(idx, center, noise):
                 new_csv_path = baseDir / csv_path.name
                 numpy.savetxt(str(new_csv_path), data, delimiter=",")
                 experiment["csv"] = str(new_csv_path)
-            for feature in experiment["features"]:
+            for feature in experiment["scores"]:
                 if "csv" in feature:
                     data = numpy.genfromtxt(feature["csv"], delimiter=",")
                     addNoise(data, center, noise)
@@ -705,21 +718,6 @@ def find_opt_poly(x, y, index):
     return root, x, y
 
 
-def fractionate(start_seq, stop_seq, times, values):
-    temp = []
-    for (start, stop) in zip(start_seq, stop_seq):
-        selected = (times >= start) & (times <= stop)
-        local_times = times[selected]
-        local_values = values[selected]
-
-        # need to use the actual start and stop times from the data no what we want to cut at
-        stop = local_times[-1]
-        start = local_times[0]
-
-        temp.append(numpy.trapz(local_values, local_times) / (stop - start))
-    return numpy.array(temp)
-
-
 def metaCSV(cache):
     repeat = int(cache.settings["repeat"])
 
@@ -1198,7 +1196,6 @@ def cleanDir(dir, hof):
 
 def finish(cache):
     sub.graph_process(cache, "Last", last=True)
-    sub.graph_spearman(cache)
 
 
 def find_outliers(data, lower_percent=10, upper_percent=90):
@@ -1470,3 +1467,15 @@ def translate_meta_min(score, cache):
         temp[:3] = 1 - temp[:3]
 
     return temp
+
+def get_bins(array, axis=None):
+    array = numpy.atleast_2d(array)
+    shape = array.shape
+    if axis:
+        length = shape[axis]
+    else:
+        length = shape[0]
+    h = 2 * scipy.stats.iqr(array, axis=axis) * length**(-1/3)
+    bins = (numpy.max(array, axis=axis) - numpy.min(array, axis=axis))/h
+    bin = scipy.stats.mstats.gmean(numpy.atleast_1d(bins))
+    return round(bin)
